@@ -29,7 +29,8 @@
 #include <meta/common.h>
 #include <meta/types.h>
 #include <pango/pango-font.h>
-#include <gdesktop-enums.h>
+#include <libcinnamon-desktop/cdesktop-enums.h>
+#include <X11/XKBlib.h>
 
 /* Keep in sync with GSettings schemas! */
 typedef enum
@@ -42,6 +43,7 @@ typedef enum
   META_PREF_ACTION_DOUBLE_CLICK_TITLEBAR,
   META_PREF_ACTION_MIDDLE_CLICK_TITLEBAR,
   META_PREF_ACTION_RIGHT_CLICK_TITLEBAR,
+  META_PREF_ACTION_SCROLL_WHEEL_TITLEBAR,
   META_PREF_AUTO_RAISE,
   META_PREF_AUTO_RAISE_DELAY,
   META_PREF_THEME,
@@ -67,7 +69,15 @@ typedef enum
   META_PREF_LIVE_HIDDEN_WINDOWS,
   META_PREF_WORKSPACES_ONLY_ON_PRIMARY,
   META_PREF_NO_TAB_POPUP,
-  META_PREF_DRAGGABLE_BORDER_WIDTH
+  META_PREF_DRAGGABLE_BORDER_WIDTH,
+  META_PREF_TILE_HUD_THRESHOLD,
+  META_PREF_RESIZE_THRESHOLD,
+  META_PREF_SNAP_MODIFIER,
+  META_PREF_LEGACY_SNAP,
+  META_PREF_INVERT_WORKSPACE_FLIP_DIRECTION,
+  META_PREF_TILE_MAXIMIZE,
+  META_PREF_PLACEMENT_MODE,
+  META_PREF_MIN_WIN_OPACITY
 } MetaPreference;
 
 typedef void (* MetaPrefsChangedFunc) (MetaPreference pref,
@@ -88,8 +98,8 @@ const char* meta_preference_to_string (MetaPreference pref);
 MetaVirtualModifier         meta_prefs_get_mouse_button_mods  (void);
 guint                       meta_prefs_get_mouse_button_resize (void);
 guint                       meta_prefs_get_mouse_button_menu  (void);
-GDesktopFocusMode           meta_prefs_get_focus_mode         (void);
-GDesktopFocusNewWindows     meta_prefs_get_focus_new_windows  (void);
+CDesktopFocusMode           meta_prefs_get_focus_mode         (void);
+CDesktopFocusNewWindows     meta_prefs_get_focus_new_windows  (void);
 gboolean                    meta_prefs_get_attach_modal_dialogs (void);
 gboolean                    meta_prefs_get_raise_on_click     (void);
 const char*                 meta_prefs_get_theme              (void);
@@ -115,9 +125,10 @@ const char*                 meta_prefs_get_terminal_command   (void);
 void                        meta_prefs_get_button_layout (MetaButtonLayout *button_layout);
 
 /* Double, right, middle click can be configured to any titlebar meta-action */
-GDesktopTitlebarAction      meta_prefs_get_action_double_click_titlebar (void);
-GDesktopTitlebarAction      meta_prefs_get_action_middle_click_titlebar (void);
-GDesktopTitlebarAction      meta_prefs_get_action_right_click_titlebar (void);
+CDesktopTitlebarAction      meta_prefs_get_action_double_click_titlebar (void);
+CDesktopTitlebarAction      meta_prefs_get_action_middle_click_titlebar (void);
+CDesktopTitlebarAction      meta_prefs_get_action_right_click_titlebar (void);
+CDesktopTitlebarScrollAction meta_prefs_get_action_scroll_wheel_titlebar (void);
 
 void meta_prefs_set_num_workspaces (int n_workspaces);
 
@@ -149,6 +160,19 @@ void     meta_prefs_set_no_tab_popup (gboolean whether);
 
 int      meta_prefs_get_draggable_border_width (void);
 
+int      meta_prefs_get_tile_hud_threshold (void);
+int      meta_prefs_get_resize_threshold (void);
+
+unsigned int *  meta_prefs_get_snap_modifier (void);
+
+gboolean meta_prefs_get_legacy_snap (void);
+
+gboolean meta_prefs_get_invert_flip_direction (void);
+
+gboolean meta_prefs_get_tile_maximize (void);
+
+gint meta_prefs_get_min_win_opacity (void);
+
 /* XXX FIXME This should be x-macroed, but isn't yet because it would be
  * difficult (or perhaps impossible) to add the suffixes using the current
  * system.  It needs some more thought, perhaps after the current system
@@ -195,8 +219,14 @@ typedef enum _MetaKeyBindingAction
   META_KEYBINDING_ACTION_ACTIVATE_WINDOW_MENU,
   META_KEYBINDING_ACTION_TOGGLE_FULLSCREEN,
   META_KEYBINDING_ACTION_TOGGLE_MAXIMIZED,
-  META_KEYBINDING_ACTION_TOGGLE_TILED_LEFT,
-  META_KEYBINDING_ACTION_TOGGLE_TILED_RIGHT,
+  META_KEYBINDING_ACTION_PUSH_TILE_LEFT,
+  META_KEYBINDING_ACTION_PUSH_TILE_RIGHT,
+  META_KEYBINDING_ACTION_PUSH_TILE_UP,
+  META_KEYBINDING_ACTION_PUSH_TILE_DOWN,
+  META_KEYBINDING_ACTION_PUSH_SNAP_LEFT,
+  META_KEYBINDING_ACTION_PUSH_SNAP_RIGHT,
+  META_KEYBINDING_ACTION_PUSH_SNAP_UP,
+  META_KEYBINDING_ACTION_PUSH_SNAP_DOWN,
   META_KEYBINDING_ACTION_TOGGLE_ABOVE,
   META_KEYBINDING_ACTION_MAXIMIZE,
   META_KEYBINDING_ACTION_UNMAXIMIZE,
@@ -236,6 +266,9 @@ typedef enum _MetaKeyBindingAction
   META_KEYBINDING_ACTION_MOVE_TO_SIDE_E,
   META_KEYBINDING_ACTION_MOVE_TO_SIDE_W,
   META_KEYBINDING_ACTION_MOVE_TO_CENTER,
+  META_KEYBINDING_ACTION_INCREASE_OPACITY,
+  META_KEYBINDING_ACTION_DECREASE_OPACITY,
+  META_KEYBINDING_ACTION_CUSTOM,
 
   META_KEYBINDING_ACTION_LAST
 } MetaKeyBindingAction;
@@ -284,13 +317,13 @@ typedef struct
    */
   GSList *bindings;
 
-  /** for keybindings that can have shift or not like Alt+Tab */
+  /* for keybindings that can have shift or not like Alt+Tab */
   gboolean      add_shift:1;
 
-  /** for keybindings that apply only to a window */
+  /* for keybindings that apply only to a window */
   gboolean      per_window:1;
 
-  /** for keybindings not added with meta_display_add_keybinding() */
+  /* for keybindings not added with meta_display_add_keybinding() */
   gboolean      builtin:1;
 } MetaKeyPref;
 
@@ -308,10 +341,8 @@ void meta_prefs_get_overlay_binding (MetaKeyCombo *combo);
 
 gboolean           meta_prefs_get_visual_bell      (void);
 gboolean           meta_prefs_bell_is_audible      (void);
-GDesktopVisualBellType meta_prefs_get_visual_bell_type (void);
+CDesktopVisualBellType meta_prefs_get_visual_bell_type (void);
+
+MetaPlacementMode meta_prefs_get_placement_mode (void);
 
 #endif
-
-
-
-
