@@ -73,6 +73,11 @@ meta_ui_init (void)
   if (!gtk_init_check (NULL, NULL))
     meta_fatal ("Unable to open X display %s\n", XDisplayName (NULL));
 
+/* We need to be able to fully trust that the window and monitor sizes
++ that Gdk reports corresponds to the X ones, so we disable the automatic
++ scale handling */
+  gdk_x11_display_set_window_scale (gdk_display_get_default (), 1);
+
   meta_stock_icons_init ();
 }
 
@@ -1024,20 +1029,6 @@ meta_stock_icons_init (void)
   g_object_unref (G_OBJECT (factory));
 }
 
-LOCAL_SYMBOL int
-meta_ui_get_drag_threshold (MetaUI *ui)
-{
-  GtkSettings *settings;
-  int threshold;
-
-  settings = gtk_widget_get_settings (GTK_WIDGET (ui->frames));
-
-  threshold = 8;
-  g_object_get (G_OBJECT (settings), "gtk-dnd-drag-threshold", &threshold, NULL);
-
-  return threshold;
-}
-
 LOCAL_SYMBOL MetaUIDirection
 meta_ui_get_direction (void)
 {
